@@ -55,7 +55,9 @@ class EnumChoice(click.Choice):
         self.choices_enum = choices_enum
 
     def convert(self, value, param, ctx):
-        name = super(EnumChoice, self).convert(value, param, ctx).replace("-", "_")
+        if isinstance(value, self.choices_enum):
+            return value
+        name = super().convert(value, param, ctx).replace("-", "_")
         return self.choices_enum[name]
 
 
@@ -83,11 +85,21 @@ def ykman_group(
 ):
     if not isinstance(connections, list):
         connections = [connections]  # Single type
-    return click.group(cls=_YkmanGroup, *args, connections=connections, **kwargs)
+    return click.group(
+        cls=_YkmanGroup,
+        *args,
+        connections=connections,
+        **kwargs,
+    )  # type: ignore
 
 
 def ykman_command(interfaces, *args, **kwargs):
-    return click.command(cls=_YkmanCommand, *args, interfaces=interfaces, **kwargs)
+    return click.command(
+        cls=_YkmanCommand,
+        *args,
+        interfaces=interfaces,
+        **kwargs,
+    )  # type: ignore
 
 
 def click_callback(invoke_on_missing=False):
@@ -206,7 +218,8 @@ def prompt_for_touch():
 def prompt_timeout(timeout=0.5):
     timer = Timer(timeout, prompt_for_touch)
     try:
-        yield timer.start()
+        timer.start()
+        yield None
     finally:
         timer.cancel()
 
